@@ -22,18 +22,11 @@
 // along with this program; if not, write to the Free Software               //
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA //
 //                                                                           //
-// $Revision:: 341                                                          $//
-// $Date:: 2013-04-08 12:21:06 +0200 (Mon, 08 Apr 2013)                     $//
+// $Revision:: 403                                                          $//
+// $Date:: 2016-05-19 16:52:05 +0200 (Thu, 19 May 2016)                     $//
 ///////////////////////////////////////////////////////////////////////////////
 
-//#ifdef HAVE_CONFIG_H
 #include <siscone/config.h>
-//#else
-//#define PACKAGE_NAME "SISCone"
-//#define VERSION "2.0.6"
-//#warning "No config.h file available, using preset values"
-//#endif
-
 #include <siscone/ranlux.h>
 #include <siscone/siscone_error.h>
 #include <siscone/defines.h>
@@ -88,35 +81,8 @@ std::ostream* CSphsiscone::_banner_ostr=&cout;
 int CSphsiscone::compute_jets(vector<CSphmomentum> &_particles, double _radius, double _f, 
 			      int _n_pass_max, double _Emin,
 			      Esplit_merge_scale _split_merge_scale){
-  // initialise random number generator
-  if (!init_done){
-    // initialise random number generator
-    siscone::ranlux_init();
-
-    // do not do this again
-    init_done=true;
-
-    // print the banner
-    if (_banner_ostr != 0){
-      (*_banner_ostr) << "#ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo" << endl;
-      (*_banner_ostr) << "#                    SISCone   version " << setw(28) << left << siscone_version() << "o" << endl;
-      (*_banner_ostr) << "#              http://projects.hepforge.org/siscone                o" << endl;
-      (*_banner_ostr) << "#                                                                  o" << endl;
-      (*_banner_ostr) << "# This is SISCone: the Seedless Infrared Safe Cone Jet Algorithm   o" << endl;
-      (*_banner_ostr) << "# SISCone was written by Gavin Salam and Gregory Soyez             o" << endl;
-      (*_banner_ostr) << "# It is released under the terms of the GNU General Public License o" << endl;
-      (*_banner_ostr) << "#                                                                  o" << endl;
-      (*_banner_ostr) << "#            !!!             WARNING            !!!                o" << endl;
-      (*_banner_ostr) << "#    This is the version of SISCone using spherical coordinates    o" << endl;
-      (*_banner_ostr) << "#                                                                  o" << endl;
-      (*_banner_ostr) << "# A description of the algorithm is available in the publication   o" << endl;
-      (*_banner_ostr) << "# JHEP 05 (2007) 086 [arXiv:0704.0292 (hep-ph)].                   o" << endl;
-      (*_banner_ostr) << "# Please cite it if you use SISCone.                               o" << endl;
-      (*_banner_ostr) << "#ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo" << endl;
-      (*_banner_ostr) << endl;
-      _banner_ostr->flush();
-    }
-  }
+  // make sure things are initialised
+  _initialise_if_needed();
 
   // run some general safety tests (NB: f will be checked in split-merge)
   if (_radius <= 0.0 || _radius >= 0.5*M_PI) {
@@ -175,6 +141,66 @@ int CSphsiscone::compute_jets(vector<CSphmomentum> &_particles, double _radius,
   return perform(_f, _Emin);
 }
 
+/*
+ * compute the jets from a given particle set doing multiple passes
+ * such pass N looks for jets among all particles not put into jets
+ * during previous passes.
+ *  - _particles   list of particles
+ *  - _radius      cone radius
+ *  - _n_pass_max  maximum number of runs
+ *  - _Emin        minimum energy of the protojets
+ *  - _ordering_scale    the ordering scale to decide which stable
+ *                       cone is removed
+ * return the number of jets found.
+ **********************************************************************/
+int CSphsiscone::compute_jets_progressive_removal(vector<CSphmomentum> &_particles, double _radius, 
+					       int _n_pass_max, double _Emin,
+					       Esplit_merge_scale _ordering_scale){
+  // make sure things are initialised
+  _initialise_if_needed();
+
+  // run some general safety tests (NB: f will be checked in split-merge)
+  if (_radius <= 0.0 || _radius >= 0.5*M_PI) {
+    ostringstream message;
+    message << "Illegal value for cone radius, R = " << _radius 
+            << " (legal values are 0<R<pi/2)";
+    throw siscone::Csiscone_error(message.str());
+  }
+
+  ptcomparison.split_merge_scale = _ordering_scale;
+  partial_clear(); // make sure some things are initialised properly
+
+  // init the split_merge algorithm with the initial list of particles
+  // this initialises particle list p_left of remaining particles to deal with
+  //
+  // this stores the "processed" particles in p_uncol_hard
+  init_particles(_particles);
+  jets.clear();
+
+  bool unclustered_left;
+  rerun_allowed = false;
+  protocones_list.clear();
+
+  do{
+    //cout << n_left << " particle left" << endl; 
+
+    // initialise stable_cone finder
+    // here we use the list of remaining particles
+    // AFTER COLLINEAR CLUSTERING !!!!!!
+    CSphstable_cones::init(p_uncol_hard);
+
+    // get stable cones (stored in 'protocones')
+    unclustered_left = get_stable_cones(_radius);
+
+    // add the hardest stable cone to the list of jets
+    if (add_hardest_protocone_to_jets(&protocones, R2, _Emin)) break;
+  
+    _n_pass_max--;
+  } while ((unclustered_left) && (n_left>0) && (_n_pass_max!=0));
+
+  // split & merge
+  return jets.size();
+}
 /*
  * recompute the jets with a different overlap parameter.
  * we use the same particles and R as in the preceeding call.
@@ -207,6 +233,43 @@ int CSphsiscone::recompute_jets(double _f, double _Emin,
 }  
 
 
+// make sure things are initialised
+void CSphsiscone::_initialise_if_needed(){
+  // initialise random number generator
+  if (init_done) return;
+
+  // initialise random number generator
+  siscone::ranlux_init();
+
+  // do not do this again
+  init_done=true;
+
+  // print the banner
+  if (_banner_ostr != 0){
+    ios::fmtflags flags_to_restore(_banner_ostr->flags());
+
+    (*_banner_ostr) << "#ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo" << endl;
+    (*_banner_ostr) << "#                    SISCone   version " << setw(28) << left << siscone_version() << "o" << endl;
+    (*_banner_ostr) << "#              http://projects.hepforge.org/siscone                o" << endl;
+    (*_banner_ostr) << "#                                                                  o" << endl;
+    (*_banner_ostr) << "# This is SISCone: the Seedless Infrared Safe Cone Jet Algorithm   o" << endl;
+    (*_banner_ostr) << "# SISCone was written by Gavin Salam and Gregory Soyez             o" << endl;
+    (*_banner_ostr) << "# It is released under the terms of the GNU General Public License o" << endl;
+    (*_banner_ostr) << "#                                                                  o" << endl;
+    (*_banner_ostr) << "#            !!!             WARNING            !!!                o" << endl;
+    (*_banner_ostr) << "#    This is the version of SISCone using spherical coordinates    o" << endl;
+    (*_banner_ostr) << "#                                                                  o" << endl;
+    (*_banner_ostr) << "# A description of the algorithm is available in the publication   o" << endl;
+    (*_banner_ostr) << "# JHEP 05 (2007) 086 [arXiv:0704.0292 (hep-ph)].                   o" << endl;
+    (*_banner_ostr) << "# Please cite it if you use SISCone.                               o" << endl;
+    (*_banner_ostr) << "#ooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo" << endl;
+    (*_banner_ostr) << endl;
+
+    _banner_ostr->flush();
+    _banner_ostr->flags(flags_to_restore);
+  }
+}
+
 // finally, a bunch of functions to access to 
 // basic information (package name, version)
 //---------------------------------------------
@@ -214,12 +277,12 @@ int CSphsiscone::recompute_jets(double _f, double _Emin,
 /* 
  * return SISCone package name.
  * This is nothing but "SISCone", it is a replacement to the
- * PACKAGE_NAME string defined in config.h and which is not
- * public by default.
+ * SISCONE_PACKAGE_NAME string defined in config.h and which is not
+ * guaranteed to be public.
  * return the SISCone name as a string
  */
 string siscone_package_name(){
-  return PACKAGE_NAME;
+  return SISCONE_PACKAGE_NAME;
 }
 
 /* 
@@ -228,7 +291,7 @@ string siscone_package_name(){
  *        (alpha, beta, devel) to mention stability status
  */
 string siscone_version(){
-  return VERSION;
+  return SISCONE_VERSION;
 }
 
 }
